import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import vis.Graph;
import vis.Vertex;

public class RecipeTester {
    public Document doc;
    public ArrayList<RecipeGraph> recipeGraphs;
    
    public void test() {
        parseXmlFile();
        readRecipes();
        compareRecipes();
//        sampleEditDistance();
//        outputRecipes();
    }
    
    public void parseXmlFile(){
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            doc = db.parse("recipes.xml");
        }catch(ParserConfigurationException pce) {
            pce.printStackTrace();
        }catch(SAXException se) {
            se.printStackTrace();
        }catch(IOException ioe) {
            ioe.printStackTrace();
        }
    }
    
    public void readRecipes() {
        recipeGraphs = new ArrayList<RecipeGraph>();
        NodeList recipeList = doc.getElementsByTagName("Recipe");
        for (int i = 0; i < recipeList.getLength(); i++) {
            Element recipe = (Element) recipeList.item(i);
            recipeGraphs.add(readRecipeGraph(recipe));
        }
    }
    
    public RecipeGraph readRecipeGraph(Element recipe) {
        RecipeGraph graph = new RecipeGraph();
        NodeList children = recipe.getChildNodes();
        for (int i = 0; i < children.getLength(); i++) {
            String typeOfNode = children.item(i).getNodeName();
            if (typeOfNode.equals("Ingredient") || typeOfNode.equals("Action")) {
                Element node = (Element) children.item(i);
                String nodeID = node.getAttribute("id");
                String label = node.getElementsByTagName("Type").item(0).getFirstChild().getNodeValue();
                graph.addNode(nodeID, label, typeOfNode);
            }
        }
        
        for (int i = 0; i < children.getLength(); i++) {
            String typeOfNode = children.item(i).getNodeName();
            if (typeOfNode.equals("Ingredient") || typeOfNode.equals("Action")) {
                Element node = (Element) children.item(i);
                String nodeID = node.getAttribute("id");
                NodeList parents = node.getElementsByTagName("Parent");
                if (typeOfNode.equals("Ingredient") && parents.getLength() == 0)
                    graph.setType(nodeID, "RawIngredient");
                for (int j = 0; j < parents.getLength(); j++) {
                    String parentID = parents.item(j).getFirstChild().getNodeValue();
                    graph.addEdge(parentID, nodeID);
                }
            }
        }
        return graph;
    }
    
    // TODO feed in the path through the command line
    public void outputRecipes(){
        for (int i = 0; i < recipeGraphs.size(); i++) {
            RecipeGraph graph = recipeGraphs.get(i);
            String dotFormattedRecipe = graph.toDotFormat();
            try {
                BufferedWriter out = new BufferedWriter(new FileWriter("C:\\Users\\Matt\\Desktop\\graphs\\recipe"+i));
                out.write(dotFormattedRecipe);
                out.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    
    public void sampleEditDistance() {
        RecipeGraph firstGraph = recipeGraphs.get(0);
        RecipeGraph secondGraph = recipeGraphs.get(2);
        RecipeGraph thirdGraph = recipeGraphs.get(6);
        
        firstGraph.removeIntermediates();
        secondGraph.removeIntermediates();
        thirdGraph.removeIntermediates();
        //System.out.println("first graph size: " + firstGraph.getSize());
        //System.out.println("second graph size: " + secondGraph.getSize());
        //long t0 = System.currentTimeMillis();
        Pair<List<int[]>, Double> pair1 = RecipeGraph.editDistance(firstGraph, secondGraph);
        List<int[]> edits1 = pair1.getFirst();
        RecipeGraph intersection = RecipeGraph.makeIntersection(firstGraph, secondGraph, edits1);
        RecipeGraph union = RecipeGraph.makeUnion(firstGraph, secondGraph, edits1);
        
        Pair<List<int[]>, Double> pair2 = RecipeGraph.editDistance(intersection, thirdGraph);
        List<int[]> edits2 = pair2.getFirst();
        
        for (int i = 0; i < edits2.size(); i++) {
            int[] e = edits2.get(i);
            // modify e[0]
        }
        
        
        RecipeGraph allThreeUnion = RecipeGraph.makeUnion(union, thirdGraph, edits2);
        
        System.out.println(union.toDotFormat());
        
        //long t1 = System.currentTimeMillis();
        //System.out.println("Time: " + (t1 - t0) + "ms");
        //System.out.println("Edit distance: " + pair.getSecond());
        
        /////////////////// Stuff to do the relative positioning
        //RecipeGraph union1 = RecipeGraph.makeUnion(firstGraph, secondGraph, edits1);
        //RecipeGraph union2 = RecipeGraph.makeUnion(firstGraph, thirdGraph, edits2);
        //RecipeGraph union3 = RecipeGraph.makeUnion(secondGraph, thirdGraph, edits3);
        
        
        //////////////////
        
    }
    
    public Graph recipeGraphToGraph(RecipeGraph rg) {
        rg.removeIntermediates();
        Graph g = new Graph();
        
        ArrayList<ArrayList<Integer>> theRecipeGraph = rg.getGraph();
        ArrayList<String> theLabels = rg.getLabels();
        ArrayList<String> theTypes = rg.getTypes();
        
        int n = theRecipeGraph.size();
        for (int i = 0; i < n; i++) {
            Vertex v = new Vertex(theLabels.get(i));
            g.addVertex(v);
            String type = theTypes.get(i);
            if (type.equals("Action")) {
                v.setColor("azure2");
            }
            if (type.equals("RawIngredient")) {
                v.setColor("bisque1");
            }
        }
        for (int i = 0; i < n; i++) {
            ArrayList<Integer> edges = theRecipeGraph.get(i);
            for (int j : edges) {
                g.addEdge(g.vertices.get(i), g.vertices.get(j), 1.5);
            }
        }
        
        return g;
    }
    
    public void compareRecipes() {
        RecipeGraph rg1 = recipeGraphs.get(0);
        Graph g1 = recipeGraphToGraph(rg1);
        RecipeGraph rg2 = recipeGraphs.get(4);
        Graph g2 = recipeGraphToGraph(rg2);
        
        Graph g = new Graph();
        for (Vertex v : g1.vertices)
            g.addVertex(v);
        for (Vertex v : g2.vertices)
            g.addVertex(v);
        
        Pair<List<int[]>, Double> pair = RecipeGraph.editDistance(rg1, rg2);
        List<int[]> edits = pair.getFirst();
        
        for (int[] edit : edits) {
            if (edit[0] != -1 && edit[1] != -1) {
                g.addEdge(g1.vertices.get(edit[0]), g2.vertices.get(edit[1]), 0.0);
            }
        }
        
        System.out.println("Computing Distances");
        g.computeDistances();
        System.out.println("Computing Node Positions");
        g.computeNodePositions();
        g1.output("recipe_0", true);
        g2.output("recipe_4", true);
    }
    
    public static void main(String[] args) {
        RecipeTester rt = new RecipeTester();
        rt.test();
    }

}
